# Copyright 2012, 2013 Marco Ceppi, Canonical Ltd.  This software is
# licensed under the GNU Affero General Public License version 3 (see
# the file LICENSE).

"""
Charm/Interface Graph Visualization.

Using thejit.org canvas graphing.
http://thejit.org/static/v20/Docs/files/Loader/Loader-js.html
"""


def graph_by_charms(charms, interfaces):
    """Force directed graph

    Per thejit loader format for graphs.

    charms with edges to required interfaces.
    interfaces with edges to provider charms.
    """
    nodes = []

    for charm in charms:
        data = {
            "summary": charm.summary,
            #"series": charm['series'],
            "i_requires": charm.i_requires,
            "i_provides": charm.i_provides,
            'label': charm.label,
            'url': charm.short_url,
        }
        node = {
            'id': charm.label,
            'adjacencies': charm.i_requires,
            'data': data}
        nodes.append(node)

    for interface in interfaces:
        node = {
            'id': interface['name'],
            #'data': dict(interface),   # Use a copy
            'adjacencies': list(
                [i.label for i in interface['provided_by']])
        }
        nodes.append(node)

    return nodes


def graph_by_interface(interfaces):
    """Hypertree data set, artifical root to interfaces to providers.
    """

    nodes = []

    for i in interfaces:
        node = {'id': i['name']}

        node['adjacencies'] = list(i['i_requires'])
        nodes.append(node)

    return nodes
