# Copyright 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).
import datetime
import logging
import time

import pymongo
import requests

from charmworld.utils import (
    configure_logging,
    get_ini,
    Timer,
)
from config import MONGO_URL
from utils import lock
from utils import LockHeld


def get_data_from_askubuntu(page):
    window = datetime.datetime.utcnow() - datetime.timedelta(30 * 6)
    window = int(time.mktime(window.timetuple()))
    url = ('https://api.stackexchange.com/2.1/questions/unanswered?' +
           'order=desc&tagged=juju&site=askubuntu&' +
           'fromdate=%s&page=%s' % (window, page))
    r = requests.get(url)
    return r.json()


def get_questions(page):
    data = get_data_from_askubuntu(page)
    results = data['items']
    if data['has_more']:
        results += get_questions(page + 1)
    return results


def get_all_questions(log):
    with Timer() as timer:
        questions = get_questions(1)
    log.debug("askubuntu query time %0.2f", timer.duration())
    return questions


def create_entries_from_questions(questions):
    entries = []
    for q in questions:
        date_modified = datetime.datetime.fromtimestamp(
            q['last_activity_date'])
        date_created = datetime.datetime.fromtimestamp(q['creation_date'])
        entry = {
            'type': 'question',
            'date_modified': date_modified,
            'date_created': date_created,
            'summary': q['title'],
            'item': q['link'],
            'status': 'Unanswered'
        }
        entries.append(entry)
    return entries


def update_question_collection(db, entries):
    db.drop_collection('answer_queue')
    if (entries != []):
        # Needs check as bulk insert of an empty list is prohibited
        db.answer_queue.insert(entries)


def main():
    configure_logging()
    log = logging.getLogger("charm.askubuntu")
    conn = pymongo.Connection(MONGO_URL, safe=True)
    db = conn['juju']
    settings = get_ini()
    questions = get_all_questions(log)
    entries = create_entries_from_questions(questions)
    try:
        with lock('askubuntu',
                  int(settings['script_lease_time']) * 60, db, log):
            update_question_collection(db, entries)
    except LockHeld, error:
        log.warn(str(error))
    log.debug('Completed job with %s entries.' % len(entries))


if __name__ == '__main__':
    main()
