# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import logging
import sys

from pyelasticsearch.exceptions import ElasticHttpError

from charmworld.charmstore import CharmStore
from charmworld.jobs.ingest import update_download_count
from charmworld.models import (
    CharmSource,
)
from charmworld.search import SearchServiceNotAvailable
from charmworld.utils import (
    configure_logging,
    get_ini,
)


def update_counts(charm_source, store, log):
    """Update the download counts on all charms in the store.

    :param charm_source: The CharmSource to read and write charms to.
    :param store: A CharmStore to use for retriving statistics.
    :param log: A log to use for information about this operation.
    """
    store_charms = charm_source.get_store_charms()
    total = store_charms.count()
    log.info("Store charms %d" % store_charms.count())
    for count, charm in enumerate(store_charms):
        update_download_count(store, charm)
        try:
            charm_source.save(charm)
        except ElasticHttpError as e:
            if 'tried to parse as object, but got EOF' not in e.error:
                raise
            log.warning(
                'Malformed charm %s cannot be indexed.' % charm['_id'])
        except SearchServiceNotAvailable as e:
            log.exception(str(e))
            return 1
        if count % 100 == 0:
            log.info("Processed %s/%s %0.1f%%" % (
                count, total, 100 * float(count) / total))
    return 0


def main():
    configure_logging()
    log = logging.getLogger("charm.stat")
    return (update_counts(CharmSource.from_settings(get_ini()), CharmStore(),
            log))


if __name__ == '__main__':
    sys.exit(main())
