# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).
import datetime
from time import mktime

from mock import patch

from charmworld.jobs.askubuntu import (
    create_entries_from_questions,
    get_questions,
)
from charmworld.testing import MongoTestBase


class FakeGet:

    def __init__(self, has_more=False):
        self.response = {'items': [1, 2, 3], 'has_more': has_more}
        self.call_count = 0

    def __call__(self, *args, **kwargs):
        self.call_count += 1
        r = dict(self.response)
        self.response['has_more'] = False
        return r


class AskUbuntuTest(MongoTestBase):

    def test_get_questions(self):
        with patch(
            'charmworld.jobs.askubuntu.get_data_from_askubuntu',
            FakeGet()) as mock:
            questions = get_questions(1)
            self.assertEqual(1, mock.call_count)
        self.assertEqual([1, 2, 3], questions)

    def test_get_questions_recursive(self):
        with patch(
            'charmworld.jobs.askubuntu.get_data_from_askubuntu',
            FakeGet(True)) as mock:
            questions = get_questions(1)
            self.assertEqual(2, mock.call_count)
        self.assertEqual([1, 2, 3, 1, 2, 3], questions)

    def test_create_entries_from_questions_empty(self):
        entries = create_entries_from_questions([])
        self.assertEqual([], entries)

    def test_create_entries_from_questions(self):
        now = datetime.datetime.utcnow()
        then = now - datetime.timedelta(1)
        questions = [{
            "question_id":345535,
            "creation_date": mktime(then.timetuple()),
            "last_activity_date": mktime(now.timetuple()),
            "score":1,
            "answer_count":1,
            "bounty_closes_date":1380123217,
            "bounty_amount":200,
            "title": "Configure a container for using it with juju",
            "tags":["juju", "cloud", "lxc"],
            "view_count":40,
            "owner":{
                "user_id":130026,
                "display_name":"jjmerelo",
                "reputation":13,
                "user_type":"registered",
                "profile_image":"https://example.com/foo.jpg",
                "link":"http://askubuntu.com/users/130026/jjmerelo",
            },
            "link":"http://example.com/questions/1/",
            "is_answered": False
        }]
        entries = create_entries_from_questions(questions)
        e = entries[0]
        self.assertEqual(1, len(entries))
        self.assertEqual('question', e['type'])
        self.assertEqual(then.date(), e['date_created'].date())
        self.assertEqual(now.date(), e['date_modified'].date())
        self.assertEqual(
            "Configure a container for using it with juju", e['summary'])
        self.assertEqual('http://example.com/questions/1/', e['item'])
        self.assertEqual('Unanswered', e['status'])
