# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from datetime import datetime
import logging

from pyelasticsearch import ElasticSearch
from requests import Response
from urlparse import (
    parse_qsl,
    urlparse,
)
from charmworld.charmstore import CharmStore
from charmworld.jobs.cstat import update_counts
from charmworld.models import CharmSource
from charmworld.search import ElasticSearchClient
from charmworld.testing import (
    factory,
    logger_buffer,
    MongoTestBase,
    temp_index_client,
)


class TestUpdateCounts(MongoTestBase):

    def test_update_counts(self):
        charm_id, charm = factory.makeCharm(self.db)
        index_client = self.use_context(temp_index_client())
        index_client.index_charm(charm)
        log = logging.getLogger("charm.stat")
        store = CharmStore()

        class FakeSession:

            @staticmethod
            def send(request):
                response = Response()
                params = dict(parse_qsl(urlparse(request.url).query))
                if ('start' in params):
                    start = datetime.strptime(params['start'], '%Y-%m-%d')
                    end = datetime.strptime(params['end'], '%Y-%m-%d')
                    days = (end - start).days
                    if days == 7:
                        response._content = ('[["2010-12-24", 4],'
                                             ' ["2010-12-25", 1]]')
                    elif days == 30:
                        response._content = ('[["2010-12-24", 10],'
                                             ' ["2010-12-24", 4],'
                                             ' ["2010-12-25", 1]]')
                    elif days == 182:
                        response._content = ('[["2010-12-23", 10],'
                                             ' ["2010-12-24", 4],'
                                             ' ["2010-12-25", 1]]')
                        response._content = ('[["2010-12-22", 100],'
                                             ' ["2010-12-23", 10],'
                                             ' ["2010-12-24", 4],'
                                             ' ["2010-12-25", 1]]')
                else:
                    response._content = '[[25]]'
                return response

        store.session = FakeSession
        update_counts(CharmSource(self.db, index_client), store, log)
        db_charm = self.db.charms.find_one({'_id': charm_id})
        self.assertEqual(5, db_charm['downloads_in_past_7_days'])
        self.assertEqual(15, db_charm['downloads_in_past_30_days'])
        self.assertEqual(115, db_charm['downloads_in_past_half_year'])
        self.assertEqual(25, db_charm['downloads'])
        index_charm = index_client.get(charm_id)
        self.assertEqual(5, index_charm['downloads_in_past_7_days'])
        self.assertEqual(15, index_charm['downloads_in_past_30_days'])
        self.assertEqual(115, index_charm['downloads_in_past_half_year'])
        self.assertEqual(25, index_charm['downloads'])

    def test_update_no_index(self):
        charm_id, charm = factory.makeCharm(self.db)
        index_client = ElasticSearchClient(
            ElasticSearch(['http://localhost:70']), 'foo')
        log = logging.getLogger("charm.stat")
        store = CharmStore()

        class FakeSession:

            @staticmethod
            def send(request):
                response = Response()
                response._content = '[["2010-12-24", 4], ["2010-12-25", 1]]'
                return response

        store.session = FakeSession
        with logger_buffer('charm.stat') as handler:
            exit_code = update_counts(CharmSource(self.db, index_client),
                                      store, log)
        self.assertEqual(40, handler.buffer[1].levelno)
        self.assertEqual('Could not connect to search server.',
                         handler.buffer[1].msg)
        self.assertEqual(2, len(handler.buffer))
        self.assertEqual(1, exit_code)
