# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from logging import getLogger

from charmtools import proof
from mock import (
    MagicMock,
    patch,
)

from charmworld.jobs.ingest import update_proof_data
from charmworld.testing import factory
from charmworld.testing import JobTestBase


class ProofTest(JobTestBase):

    def test_no_proof_for_deleted_branch(self):
        # If the branch of a charm is deleted from Launchpad, the proof
        # job does nothing.
        charm = factory.get_charm_json(branch_deleted=True)
        proofer = MagicMock()
        with patch.object(proof, 'run', new_method=proofer):
            update_proof_data(charm, getLogger('charm.proof'))
        self.assertFalse(proofer.called)

    def test_update_proof_data(self):
        # update_proof_data() adds the field 'proof' to the charm data.
        charm = factory.get_charm_json(with_real_files=True)
        update_proof_data(charm, getLogger('charm.proof'))
        self.assertEqual(
            {
                'e': [
                    ' Unknown root metadata field (dotted.name)',
                    ' no copyright file',
                    ' relation ssl_website is not a map',
                    ' relation sql_db is not a map',
                ],
                'w': [
                    ' Metadata is missing categories.',
                    ' website-relation-changed not executable',
                    ' database-relation-changed not executable',
                    ' install not executable',
                    ' start not executable',
                    ' stop not executable',
                    ' config-changed not executable',
                    (' config.yaml: option error-email has an invalid '
                     'type (str)'),
                ],
                'i': [
                    ' relation ssl_website has no hooks',
                    ' relation sql_db has no hooks',
                ],
            },
            charm['proof'])
