"""Helpers for dealing with deployer files."""
from copy import deepcopy
from deployer.deployment import Deployment
import yaml

from charmworld.lib.proof import ProofError


def parse_deployer(data, format='yaml'):
    try:
        return yaml.safe_load(data)
    except yaml.YAMLError, exc:
        raise ProofError(
            [
                data,
                str(exc)
            ],
            'Could not parse the yaml provided.'
        )


class CharmworldDeployment(Deployment):
    """A fake transitional Deployment object from the deployer.


    """

    def __init__(self, data):
        """

        :param: data the parsed bundle information.

        """
        self.data = data


# XXX: This is copied old deployer code needed to make the models
# get_flattened_deployment work properly. The goal is to replace that call
# with proper usage of the deployer config code that does this for us. It's
# copied for now to allow updating the version of the deployer from 0.1.1 to
# 0.2.5.
class CycleDetected(Exception):
    """An inheritance cycle was detected in a deployment config."""

    def __init__(self, key):
        super(CycleDetected, self).__init__(
            'An inheritance cycle was detected involving deployment "%s"'
            % key)


# Utils from deployer 1
def relations_combine(onto, source):
    target = deepcopy(onto)
    # Support list of relations targets
    if isinstance(onto, list) and isinstance(source, list):
        target.extend(source)
        return target
    for (key, value) in source.items():
        if key in target:
            if isinstance(target[key], dict) and isinstance(value, dict):
                target[key] = relations_combine(target[key], value)
            elif isinstance(target[key], list) and isinstance(value, list):
                target[key] = list(set(target[key] + value))
        else:
            target[key] = value
    return target


def dict_merge(onto, source):
    target = deepcopy(onto)
    for (key, value) in source.items():
        if key == 'relations' and key in target:
            target[key] = relations_combine(target[key], value)
        elif (key in target and isinstance(target[key], dict) and
                isinstance(value, dict)):
            target[key] = dict_merge(target[key], value)
        else:
            target[key] = value
    return target


def inherits(d):
    parents = d.get('inherits', ())
    if isinstance(parents, basestring):
        parents = [parents]
    return parents


def get_flattened_deployment(data, key, seen=None):
    """This flattens the inheritance hierarchy.

    This is provided as a public API for tools, such as Charmworld, that need
    to parse deployment configs.
    """
    if seen is None:
        seen = set()
    elif key in seen:
        raise CycleDetected(key)
    deploy_data = data[key]
    if not 'inherits' in deploy_data:
        return deploy_data
    parents = inherits(deploy_data)
    for parent_name in parents:
        parent_data = get_flattened_deployment(
            data, parent_name, seen | set([key]))
        parents.extend(inherits(parent_data))
        deploy_data = dict_merge(deploy_data, parent_data)
    deploy_data['inherits'] = parents
    return deploy_data
