# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from ConfigParser import ConfigParser
import sys


def merge_configs(merged_file, default_config_file, custom_config_file=None,
                  err=sys.stderr):
    """Read the ini files default_config_file and custom_config_file and
    write the settings from both files to merged_file.

    Settings in custom_config_file override those from default_config_file.

    If custom_config_file contains definitions that do not appear in
    default_config_file, a warning is printed.
    """
    default_config = ConfigParser()
    default_config.readfp(open(default_config_file))

    if custom_config_file is not None:
        custom_config = ConfigParser()
        custom_config.read(custom_config_file)

        default_sections = set(default_config.sections())
        custom_sections = set(custom_config.sections())
        new_sections = custom_sections.difference(default_sections)
        if len(new_sections) > 0:
            print >>err, (
                "Warning: %s defines sections that do not appear in %s: %s"
                % (custom_config_file, default_config_file,
                   ', '.join(sorted(new_sections))))

        for section in sorted(default_sections):
            if section not in custom_sections:
                continue
            default_options = set(default_config.options(section))
            custom_options = set(custom_config.options(section))
            new_options = custom_options.difference(default_options)
            if len(new_options) > 0:
                for option in sorted(new_options):
                    print >>err, (
                        "Warning: Section %s, option %s only defined in %s, "
                        "not in %s"
                        % (section, option, custom_config_file,
                           default_config_file))
        default_config.read(custom_config_file)

    with open(merged_file, 'w') as output:
        output.write(
            '# This file is automatically generated by "make config". Do not\n'
            '# change it manually. Put custom changes instead into the file\n'
            '# custom.ini; generally useful changes should be applied to\n'
            '# default.ini.\n')
        default_config.write(output)


if __name__ == '__main__':
    merge_configs(*sys.argv[1:])
