# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import logging
from pyramid.httpexceptions import HTTPFound
from pyramid.security import remember
from pyramid.security import forget
from pyramid.view import view_config

from charmworld.models import UserMgr


LOG = logging.getLogger(__name__)


@view_config(route_name="logout")
def logout(request):
    headers = forget(request)
    request.session.delete()
    return HTTPFound(location=request.route_url('home'),
                     headers=headers)


@view_config(
    route_name="auth_callback")
def auth_callback(request):
    """Successfully logging in is remembered.

    """
    headers = {}
    params = dict(request.params)

    # Can be an openid.mode = 'cancel' from the SSO.
    mode = params.get('openid.mode')
    if mode == u'cancel':
        # The user didn't login but canceled during walk through.
        LOG.error('OpenID Cancel')
        # Return them to the auth page for now.
        return HTTPFound(location=request.route_url('home'))
    elif mode == u'id_res':
        # Successful login is openid.mode = 'id_res'

        # Attempt to find the user in mongo, else create it.
        found = UserMgr.find_one(request.db, params.get('openid.claimed_id'))

        if not found:
            user = UserMgr.from_sso(params)
            user.save(request.db)
        else:
            user = found
            user.update_from_sso(params)
            user.save(request.db)

        # Add the authn token to the cookie via headers with an expiration XX
        # seconds out.
        headers = remember(
            request,
            user.userid,
            max_age=request.registry.settings.get('login_cookie_timeout'))
    else:
        LOG.error('Unexpected mode from openid: ' + str(mode))

    request.session.delete()
    return HTTPFound(location=request.route_url('home'),
                     headers=headers)
