# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from deform import (
    Form,
    ValidationFailure,
)
from pyramid.httpexceptions import HTTPFound
from pyramid.view import view_config

from charmworld.forms.featured import CharmFeaturedness
from charmworld.models import FeaturedSource
from charmworld.views.helpers import (
    find_charm,
    find_bundle,
    found,
)


def featured_edit(request, item, kind, url):
    """Back end for featuring an "item" which can be a charm or bundle."""
    form = Form(CharmFeaturedness(), buttons=('submit',))
    form.css_class = 'form-horizontal well'
    featured_source = FeaturedSource.from_db(request.db)

    if 'submit' in request.POST:
        try:
            data = request.POST.items()
            valid_data = form.validate(data)
            if valid_data['is_featured']:
                featured_source.set_featured(item, kind)
            else:
                featured_source.clear_featured(item, kind)
            # Send the user back to the item's main page after saving.
            return HTTPFound(url)
        except ValidationFailure as form:
            valid_data = {}
    else:
        valid_data = {
            'is_featured': featured_source.is_featured(item, kind)}

    return {
        'form': form.render(valid_data),
        'values': valid_data,
    }


@view_config(
    route_name="charm-featured-edit",
    renderer='charmworld:templates/qa_edit.pt',
    permission='edit',
)
def charm_featured_edit(request):
    """Back end for featuring a charm."""
    charm = find_charm(request, promulgated=True)
    found(charm)
    series = request.matchdict['series']
    charm_name = request.matchdict['charm']
    url = request.route_url('charm', series=series, charm=charm_name)
    return featured_edit(request, charm, 'charm', url)


@view_config(
    route_name="bundle-featured-edit",
    renderer='charmworld:templates/qa_edit.pt',
    permission='edit',
)
def bundle_featured_edit(request):
    """Back end for featuring a bundle."""
    bundle = find_bundle(request, promulgated=True)
    # Assert that the bundle was found, generating a 404 if not.
    found(bundle)
    basket = request.matchdict.get('basket')
    basket_name = request.matchdict.get('bundle')
    # Since there is not yet a web page for promulgated bundles, we redirect
    # to the owner's page for the bundle.
    # TODO Change this to the promulgated bundle page when they exist.
    url = request.route_url(
        'personal-bundle', owner=bundle.owner, basket=basket,
        bundle=basket_name)
    return featured_edit(request, dict(bundle), 'bundle', url)
