# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import openid.oidutil

from charmworld.testing import factory
from charmworld.testing import login_request
from charmworld.testing import WebTestBase


openid.oidutil.log = lambda message, level=0: None


class TestAuthViews(WebTestBase):

    def test_post_auth(self):
        """After successful SSO auth you're returned home."""
        resp = self.app.get(
            '/auth_callback',
            params=factory.makeSSOResponse(),
            status=302)
        resp = resp.follow()
        self.assertEqual("/", resp.request.path)
        self.assertIn('auth_tkt', resp.request.cookies)

    def test_sso_cancel(self):
        """If you cancel during SSO auth you're returned to /home"""
        resp = self.app.get(
            '/auth_callback',
            params=factory.makeSSOResponse(sso_type='cancel'),
            status=302)

        resp = resp.follow()
        self.assertEqual("/", resp.request.path)
        self.assertNotIn('auth_tkt', resp.request.cookies)


class TestAuthLoginLogout(WebTestBase):
    """The nav item for Login/Logout matches current user status."""

    def test_starts_with_login(self):
        """Without being logged in you get the login nav link."""
        resp = self.app.get('/', status=200)

        body = resp.body
        self.assertIn("/login/openid", body)
        self.assertNotIn("/logout", body)

    def test_after_login_see_logout(self):
        """Once logged in you get the logout nav link."""
        with login_request(self.app) as resp:
            resp.goto('/', status=200)
            body = resp.body
            self.assertNotIn("/login/openid", body)
            self.assertIn("/logout", body)

    def test_login_route(self):
        path = '/login/openid'
        self.app.get(path, status=200)

    def test_logout_route(self):
        path = '/logout'
        self.app.get(path, status=302)
