# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from mock import patch
from pyelasticsearch import ElasticSearch

from charmworld.search import (
    ElasticSearchClient,
    IndexNotReady,
)
from charmworld.testing import (
    ViewTestBase,
    WebTestBase,
)
from charmworld.views import search


RETURN_VAL = {
    'results': [],
    'search_time': 0.0,
    'result_total': 0,
    'matches': 0,
}


class SearchTests(ViewTestBase):

    @patch('charmworld.views.search.do_search',
           autospec=True, return_value=RETURN_VAL)
    def test_search(self, mock):
        terms = {"search_text": 'foo'}
        keys = ['result_total', 'results', 'search_time']
        request = self.getRequest(params=terms)
        response = search.search(request)

        mock.assert_called_with(request)
        self.assertEqual(keys, sorted(response.keys()))
        self.assertEqual(0, response['result_total'])
        self.assertEqual([], response['results'])
        self.assertEqual(0.0, response['search_time'])

    @patch('charmworld.views.search.do_search',
           autospec=True, side_effect=IndexNotReady)
    def test_search_503(self, mock):
        terms = {"search_text": 'foo'}
        request = self.getRequest(params=terms)
        # render_to_response requires the request to be much more like a real
        # request, with scheme and static stuff handling, so mock it out.
        with patch('charmworld.views.search.render_to_response') as render:
            response = search.search(request)
        render.assert_called_with('../templates/search-not-ready.pt', {},
                                  request=request)
        self.assertEqual(503, response.status_code)

    def test_search_503_on_connection_error(self):
        terms = {"search_text": 'foo'}
        request = self.getRequest(params=terms)
        request.index_client = ElasticSearchClient(
            ElasticSearch(['http://localhost:70']), 'foo')
        # render_to_response requires the request to be much more like a real
        # request, with scheme and static stuff handling, so mock it out.
        with patch('charmworld.views.search.render_to_response') as render:
            response = search.search(request)
        render.assert_called_with('../templates/search-not-ready.pt', {},
                                  request=request)
        self.assertEqual(503, response.status_code)


class SearchJSONTests(WebTestBase):

    def test_search_json_obsolete(self):
        obsolete_message = 'no longer supported'
        resp = self.app.get('/search/json', status=410)
        self.assertIn(obsolete_message, resp.body)
