import json
from webob import Response


def json_response(status, value, headers=[]):
    """Return a JSON API response.

    :param status: The HTTP status code to use.
    :param value: A json-serializable value to use as the body, or None for
        no body.
    """
    # If the value is an object, then it must be representable as a mapping.
    if value is None:
        body = ''
    else:
        while True:
            try:
                body = json.dumps(value, sort_keys=True, indent=2)
            except TypeError:
                # If the serialization wasn't possible, and the value wasn't
                # already a dictionary, convert it to one and try again.
                if not isinstance(value, dict):
                    value = dict(value)
                    continue
            break

    return Response(body,
                    headerlist=[
                        ('Content-Type', 'application/json'),
                        ('Access-Control-Allow-Origin', '*'),
                        ('Access-Control-Allow-Headers', 'X-Requested-With'),
                    ] + headers,
                    status_code=status)
